﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Should.Fluent;

namespace DocumentCache.Tests
{
    [TestClass]
    public class DocumentDatabaseOutputCacheProviderTests
    {
        DocumentDatabaseOutputCacheProvider _provider;

        [TestInitialize]
        public void BeforeTest()
        {
            _provider = new DocumentDatabaseOutputCacheProvider();
        }

        [TestMethod]
        public void should_save_new_object_to_cache()
        {
            string key = "12434";

            _provider.Add(key, "Hello MongoDB", DateTime.UtcNow.AddSeconds(100));
            var obj = _provider.Get(key);

            obj.Should().Equal("Hello MongoDB");

            _provider.Remove(key);
        }

        [TestMethod]
        public void should_get_existing_object_from_cache()
        {
            string key = "98762";

            _provider.Add(key, "Saved Item", DateTime.UtcNow.AddSeconds(100));
            var obj = _provider.Get(key);

            obj.Should().Equal("Saved Item");

            _provider.Remove(key);
        }

        [TestMethod]
        public void should_not_get_non_existent_object_from_cache()
        {
            string key = "90877000";

            var obj = _provider.Get(key);

            obj.Should().Be.Null();
        }

        [TestMethod]
        public void should_not_get_expired_object_from_cache()
        {
            string key = "98734234";

            _provider.Add(key, "This has expired already!", DateTime.UtcNow.AddSeconds(-10));
            var obj = _provider.Get(key);

            obj.Should().Be.Null();
        }

        [TestMethod]
        public void add_should_not_update_existing_item_in_cache()
        {
            string key = "9873";

            _provider.Add(key, "Existing Item", DateTime.UtcNow.AddSeconds(100));
            _provider.Add(key, "Updated Item", DateTime.UtcNow.AddSeconds(100));
            var obj = _provider.Get(key);

            obj.Should().Equal("Existing Item");
        }

        [TestMethod]
        public void add_should_update_expired_item_in_cache()
        {
            string key = "564536787";

            _provider.Add(key, "This has expired already!", DateTime.UtcNow.AddSeconds(-10));
            _provider.Add(key, "This hasn't", DateTime.UtcNow.AddSeconds(10));
            var obj = _provider.Get(key);

            obj.Should().Equal("This hasn't");
        }

        [TestMethod]
        public void set_should_update_existing_object_from_cache()
        {
            string key = "45645";

            _provider.Add(key, "New Document", DateTime.UtcNow.AddSeconds(100));
            _provider.Set(key, "Updated Document", DateTime.UtcNow.AddSeconds(100));
            var obj = _provider.Get(key);

            obj.Should().Equal("Updated Document");

            _provider.Remove(key);
        }

        [TestMethod]
        public void should_remove_valid_object_from_cache()
        {
            string key = "08977";

            _provider.Add(key, "Document to be deleted", DateTime.UtcNow.AddSeconds(100));
            _provider.Remove(key);

            var obj = _provider.Get(key);

            obj.Should().Be.Null();
        }

        [TestCleanup]
        public void Cleanup()
        {
            _provider.Dispose();
        }
    }
}
